@@ -22,7 +22,6 @@
 
 #include "common/log.h"
 #include "common/nalreader.h"
-#include "vaapidecoder_factory.h"
 
 #include "vaapi/vaapiptrs.h"
 #include "vaapi/vaapicontext.h"
@@ -1819,6 +1818,20 @@ YamiStatus VaapiDecoderH264::decodeNalu(NalUnit* nalu)
     return status;
 }
 
+void VaapiDecoderH264::flush(void)
+{
+    decodeCurrent();
+    m_dpb.flush();
+    m_newStream = true;
+    m_endOfStream = false;
+    m_endOfSequence = false;
+    m_currPic.reset();
+    m_prevPic.reset();
+    m_currSurface.reset();
+    m_contextChanged = false;
+    VaapiDecoderBase::flush();
+}
+
 YamiStatus VaapiDecoderH264::decode(VideoDecodeBuffer* buffer)
 {
     if (!buffer || !buffer->data) {
@@ -1827,6 +1840,10 @@ YamiStatus VaapiDecoderH264::decode(VideoDecodeBuffer* buffer)
         m_newStream = true;
         m_endOfStream = false;
         m_endOfSequence = false;
+        m_currPic.reset();
+        m_prevPic.reset();
+        m_currSurface.reset();
+        m_contextChanged = false;
         return YAMI_SUCCESS;
     }
     m_currentPTS = buffer->timeStamp;
@@ -1846,7 +1863,4 @@ YamiStatus VaapiDecoderH264::decode(VideoDecodeBuffer* buffer)
     return YAMI_SUCCESS;
 }
 
-const bool VaapiDecoderH264::s_registered
-    = VaapiDecoderFactory::register_<VaapiDecoderH264>(YAMI_MIME_AVC)
-      && VaapiDecoderFactory::register_<VaapiDecoderH264>(YAMI_MIME_H264);
 }
