@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2014 Intel Corporation. All rights reserved.
+ * Copyright (C) 2013-2016 Intel Corporation. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,7 +14,6 @@
  * limitations under the License.
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -22,7 +21,6 @@
 #include "codecparsers/h265Parser.h"
 #include "common/log.h"
 #include "common/nalreader.h"
-#include "vaapidecoder_factory.h"
 
 #include "vaapi/vaapiptrs.h"
 #include "vaapi/vaapicontext.h"
@@ -911,12 +909,66 @@ bool VaapiDecoderH265::fillSlice(const PicturePtr& picture,
     return true;
 }
 
+#define CHECK(v, expect)                                                      \
+    do {                                                                      \
+        if (v != expect) {                                                    \
+            ERROR("the value of %s is %d,  not equals to %d", #v, v, expect); \
+            return VAProfileNone;                                             \
+        }                                                                     \
+    } while (0)
+
+#define CHECK_RANGE(v, min, max)                                \
+    do {                                                        \
+        if (v < min || v > max) {                               \
+            ERROR("%s is %d, not in [%d,%d]", #v, v, min, max); \
+            return VAProfileNone;                               \
+        }                                                       \
+    } while (0)
+
+VAProfile VaapiDecoderH265::getVaProfile(const SPS* const sps)
+{
+    if (sps->profile_tier_level.general_profile_idc == 0
+        || sps->profile_tier_level.general_profile_compatibility_flag[0] == 1) {
+        //general_profile_idc = 0 is not defined in spec, but some stream have this
+        //we treat it as Main profile
+        CHECK(sps->chroma_format_idc, 1);
+        CHECK(sps->bit_depth_luma_minus8, 0);
+        CHECK(sps->bit_depth_chroma_minus8, 0);
+        return VAProfileHEVCMain;
+    }
+
+    if (sps->profile_tier_level.general_profile_idc == 1
+        || sps->profile_tier_level.general_profile_compatibility_flag[1] == 1) {
+        //A.3.2, but we only check some important values
+        CHECK(sps->chroma_format_idc, 1);
+        CHECK(sps->bit_depth_luma_minus8, 0);
+        CHECK(sps->bit_depth_chroma_minus8, 0);
+        return VAProfileHEVCMain;
+    }
+    if (sps->profile_tier_level.general_profile_idc == 2
+        || sps->profile_tier_level.general_profile_compatibility_flag[2] == 1) {
+        //A.3.3, but we only check some important values
+        CHECK(sps->chroma_format_idc, 1);
+        CHECK_RANGE(sps->bit_depth_luma_minus8, 0, 2);
+        CHECK_RANGE(sps->bit_depth_chroma_minus8, 0, 2);
+        return VAProfileHEVCMain10;
+    }
+    ERROR("unsupported profile %d", sps->profile_tier_level.general_profile_idc);
+    return VAProfileNone;
+}
+#undef CHECK
+#undef CHECK_RANGE
+
 YamiStatus VaapiDecoderH265::ensureContext(const SPS* const sps)
 {
     uint8_t surfaceNumber = sps->sps_max_dec_pic_buffering_minus1[0] + 1;
+    VAProfile profile = getVaProfile(sps);
+    if (profile == VAProfileNone)
+        return YAMI_UNSUPPORTED;
     if (m_configBuffer.surfaceWidth < sps->width
-        || m_configBuffer.surfaceHeight <  sps->height
-        || m_configBuffer.surfaceNumber < surfaceNumber) {
+        || m_configBuffer.surfaceHeight < sps->height
+        || m_configBuffer.surfaceNumber < surfaceNumber
+        || m_configBuffer.profile != profile) {
         INFO("frame size changed, reconfig codec. orig size %d x %d, new size: %d x %d",
                 m_configBuffer.width, m_configBuffer.height, sps->width, sps->height);
         YamiStatus status = VaapiDecoderBase::terminateVA();
@@ -927,7 +979,8 @@ YamiStatus VaapiDecoderH265::ensureContext(const SPS* const sps)
         m_configBuffer.surfaceWidth = sps->width;
         m_configBuffer.surfaceHeight =sps->height;
         m_configBuffer.flag |= HAS_SURFACE_NUMBER;
-        m_configBuffer.profile = VAProfileHEVCMain;
+        m_configBuffer.profile = profile;
+        m_configBuffer.fourcc = (profile == VAProfileHEVCMain10) ? YAMI_FOURCC_P010 : YAMI_FOURCC_NV12;
         m_configBuffer.surfaceNumber = surfaceNumber;
         status = VaapiDecoderBase::start(&m_configBuffer);
         if (status != YAMI_SUCCESS)
@@ -1083,6 +1136,18 @@ YamiStatus VaapiDecoderH265::decodeNalu(NalUnit* nalu)
     return status;
 }
 
+void VaapiDecoderH265::flush(void)
+{
+    decodeCurrent();
+    m_dpb.flush();
+    m_prevPicOrderCntMsb = 0;
+    m_prevPicOrderCntLsb = 0;
+    m_newStream = true;
+    m_endOfSequence = false;
+    m_prevSlice.reset(new SliceHeader());
+    VaapiDecoderBase::flush();
+}
+
 YamiStatus VaapiDecoderH265::decode(VideoDecodeBuffer* buffer)
 {
     if (!buffer || !buffer->data) {
@@ -1092,6 +1157,7 @@ YamiStatus VaapiDecoderH265::decode(VideoDecodeBuffer* buffer)
         m_prevPicOrderCntLsb = 0;
         m_newStream = true;
         m_endOfSequence = false;
+        m_prevSlice.reset(new SliceHeader());
         return YAMI_SUCCESS;
     }
     m_currentPTS = buffer->timeStamp;
@@ -1119,7 +1185,7 @@ bool VaapiDecoderH265::decodeHevcRecordData(uint8_t* buf, int32_t bufSize)
     }
     /*
      * Some hvcC format don't used buf[0]==1 as flag, so now used the
-     * (buf[0] || buf[1] || buf[2] > 1) as hvcC condition. 
+     * (buf[0] || buf[1] || buf[2] > 1) as hvcC condition.
      */
     if (!(buf[0] || buf[1] || buf[2] > 1)) { //annexb format
         VideoDecodeBuffer buffer;
@@ -1163,8 +1229,5 @@ bool VaapiDecoderH265::decodeHevcRecordData(uint8_t* buf, int32_t bufSize)
     return true;
 }
 
-const bool VaapiDecoderH265::s_registered =
-    VaapiDecoderFactory::register_<VaapiDecoderH265>(YAMI_MIME_H265);
-
 }
 
