@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2014 Intel Corporation. All rights reserved.
+ * Copyright (C) 2013-2016 Intel Corporation. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,7 +28,6 @@
 #include "vaapi/vaapidisplay.h"
 #include "vaapicodedbuffer.h"
 #include "vaapiencpicture.h"
-#include "vaapiencoder_factory.h"
 #include <algorithm>
 #include <math.h>
 
@@ -42,9 +41,6 @@ using YamiParser::BitWriter;
 using std::list;
 using std::vector;
 
-/* Define the maximum IDR period */
-#define MAX_IDR_PERIOD 2000
-
 #define LEVEL51_MAX_MBPS 983040
 #define H264_FRAME_FR 172
 #define H264_MIN_CR 2
@@ -866,6 +862,7 @@ VaapiEncoderH264::VaapiEncoderH264()
     m_videoParamAVC.deblockBetaOffsetDiv2 = 2;
     m_videoParamAVC.temporalLayerNum = 1;
     m_videoParamAVC.priorityId = 0;
+    m_videoParamAVC.enablePrefixNalUnit = false;
     m_maxOutputBuffer = H264_MIN_TEMPORAL_GOP;
 }
 
@@ -997,6 +994,17 @@ void VaapiEncoderH264::checkSvcTempLimitaion()
 
 void VaapiEncoderH264::resetParams ()
 {
+    if (m_videoParamCommon.enableLowPower) {
+#if VA_CHECK_VERSION(0, 39, 2)
+        if (ipPeriod() > 1) {
+            WARNING("Low power mode can not support B frame encoding");
+            m_videoParamCommon.ipPeriod = 1; // without B frame
+        }
+        m_entrypoint = VAEntrypointEncSliceLP;
+#else
+        ERROR("For AVC lowpower mode, please make sure libva version >= 0.39.2");
+#endif
+    }
 
     m_levelIdc = level();
 
@@ -1005,6 +1013,10 @@ void VaapiEncoderH264::resetParams ()
 
     m_temporalLayerNum = m_videoParamAVC.temporalLayerNum;
 
+    // enable prefix nal unit for simulcast or svc-t
+    if (m_temporalLayerNum > 1 || m_videoParamAVC.priorityId)
+        m_videoParamAVC.enablePrefixNalUnit = true;
+
     checkProfileLimitation();
     checkSvcTempLimitaion();
 
@@ -1025,9 +1037,6 @@ void VaapiEncoderH264::resetParams ()
 
     m_keyPeriod = intraPeriod() * (m_videoParamAVC.idrInterval + 1);
 
-    if (m_keyPeriod > MAX_IDR_PERIOD)
-        m_keyPeriod = MAX_IDR_PERIOD;
-
     if (initQP() < minQP())
         initQP() = minQP();
 
@@ -1662,11 +1671,12 @@ bool VaapiEncoderH264::addPackedPrefixNalUnit(const PicturePtr& picture) const
 
     bit_writer_write_trailing_bits(&bs);
 
+    uint32_t codedBits = bs.getCodedBitsCount();
     uint8_t* codedData = bs.getBitWriterData();
-    ASSERT(codedData);
+    ASSERT(codedData && codedBits);
 
     if (!picture->addPackedHeader(VAEncPackedHeaderRawData, codedData,
-                                  bs.getCodedBitsCount())) {
+                                  codedBits)) {
         ret = false;
     }
 
@@ -1710,7 +1720,7 @@ bool VaapiEncoderH264::addPackedSliceHeader(
 
     /* frame_mbs_only_flag == 1 */
     if (!m_seqParam->seq_fields.bits.frame_mbs_only_flag) {
-        ERROR("interlace unspported");
+        ERROR("interlace unsupported");
         return false;
     }
 
@@ -1723,7 +1733,7 @@ bool VaapiEncoderH264::addPackedSliceHeader(
             m_seqParam->seq_fields.bits.log2_max_pic_order_cnt_lsb_minus4 + 4);
         /* pic_order_present_flag == 0 */
     } else {
-        ERROR("POC type unspported");
+        ERROR("POC type unsupported");
         return false;
     }
 
@@ -1829,11 +1839,12 @@ bool VaapiEncoderH264::addPackedSliceHeader(
         bs.writeToBytesAligned(true);
     }
 
+    uint32_t codedBits = bs.getCodedBitsCount();
     uint8_t* codedData = bs.getBitWriterData();
-    ASSERT(codedData);
+    ASSERT(codedData && codedBits);
 
     if (!picture->addPackedHeader(VAEncPackedHeaderSlice, codedData,
-                                  bs.getCodedBitsCount())) {
+                                  codedBits)) {
         ret = false;
     }
 
@@ -1917,7 +1928,8 @@ bool VaapiEncoderH264::addSliceHeaders (const PicturePtr& picture) const
         /* set calculation for next slice */
         lastMbIndex += curSliceMbs;
 
-        if (!addPackedPrefixNalUnit(picture))
+        if (m_videoParamAVC.enablePrefixNalUnit
+            && !addPackedPrefixNalUnit(picture))
             return false;
         if (!addPackedSliceHeader(picture, sliceParam))
             return false;
@@ -2004,8 +2016,4 @@ YamiStatus VaapiEncoderH264::encodePicture(const PicturePtr& picture)
     return YAMI_SUCCESS;
 }
 
-const bool VaapiEncoderH264::s_registered =
-    VaapiEncoderFactory::register_<VaapiEncoderH264>(YAMI_MIME_AVC)
-    && VaapiEncoderFactory::register_<VaapiEncoderH264>(YAMI_MIME_H264);
-
 }
