@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Intel Corporation. All rights reserved.
+ * Copyright (C) 2014-2016 Intel Corporation. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,7 +28,6 @@
 #include "vaapi/vaapidisplay.h"
 #include "vaapicodedbuffer.h"
 #include "vaapiencpicture.h"
-#include "vaapiencoder_factory.h"
 #include <algorithm>
 
 namespace YamiMediaCodec{
@@ -42,9 +41,6 @@ using std::list;
 using std::vector;
 using std::deque;
 
-/* Define the maximum IDR period */
-#define MAX_IDR_PERIOD 2000
-
 #define HEVC_NAL_START_CODE 0x000001
 
 #define HEVC_SLICE_TYPE_I            2
@@ -156,6 +152,7 @@ static uint8_t hevc_get_profile_idc(VideoProfile profile)
         break;
     case PROFILE_H265_MAIN10:
         idc = 2;
+        break;
     default:
         assert(0);
     }
@@ -234,8 +231,6 @@ static void profile_tier_level(
 
     unsigned char vps_general_level_idc = seq->general_level_idc * 3;
 
-    general_profile_compatibility_flag[1] = general_profile_compatibility_flag[2] = 1;
-
     if (profile_present_flag) {
         /* general_profile_space */
         bitwriter->writeBits(0, 2);
@@ -245,6 +240,7 @@ static void profile_tier_level(
         bitwriter->writeBits(seq->general_profile_idc, 5);
 
         /* general_profile_compatibility_flag. Only the bit corresponding to profile_idc is set */
+        general_profile_compatibility_flag[seq->general_profile_idc] = 1;
         for (i = 0; i < N_ELEMENTS(general_profile_compatibility_flag); i++) {
             bitwriter->writeBits(general_profile_compatibility_flag[i], 1);
         }
@@ -899,6 +895,8 @@ void VaapiEncoderHEVC::resetParams ()
 {
 
     m_levelIdc = level();
+    if (10 == m_videoParamCommon.bitDepth)
+        m_videoParamCommon.profile = VAProfileHEVCMain10;
     m_profileIdc = hevc_get_profile_idc(profile());
 
     m_numSlices = 1;
@@ -941,9 +939,6 @@ void VaapiEncoderHEVC::resetParams ()
 
     m_keyPeriod = intraPeriod() * (m_videoParamAVC.idrInterval + 1);
 
-    if (m_keyPeriod > MAX_IDR_PERIOD)
-        m_keyPeriod = MAX_IDR_PERIOD;
-
     if (minQP() > initQP() ||
             (rateControlMode()== RATE_CONTROL_CQP && minQP() < initQP()))
         minQP() = initQP();
@@ -1372,9 +1367,10 @@ bool VaapiEncoderHEVC::fill(VAEncSequenceParameterBufferHEVC* seqParam) const
     /* separate color plane_flag*/
     seqParam->seq_fields.bits.separate_colour_plane_flag = 0;
     /* bit_depth_luma_minus8. Only 0 is supported for main profile */
-    seqParam->seq_fields.bits.bit_depth_luma_minus8 = 0;
+    seqParam->seq_fields.bits.bit_depth_luma_minus8 = m_videoParamCommon.bitDepth - 8;
     /* bit_depth_chroma_minus8. Only 0 is supported for main profile*/
-    seqParam->seq_fields.bits.bit_depth_chroma_minus8 = 0;
+    seqParam->seq_fields.bits.bit_depth_chroma_minus8 = m_videoParamCommon.bitDepth - 8;
+
     /* scaling_list_enabled_flag. Use the default value  */
     seqParam->seq_fields.bits.scaling_list_enabled_flag = 0;
     /* strong_intra_smoothing_enabled_flag. Not use the bi-linear interpolation */
@@ -1765,8 +1761,4 @@ YamiStatus VaapiEncoderHEVC::encodePicture(const PicturePtr& picture)
     return YAMI_SUCCESS;
 }
 
-const bool VaapiEncoderHEVC::s_registered =
-    VaapiEncoderFactory::register_<VaapiEncoderHEVC>(YAMI_MIME_HEVC)
-    && VaapiEncoderFactory::register_<VaapiEncoderHEVC>(YAMI_MIME_H265);
-
 }
