@@ -37,11 +37,11 @@
 #include <va/va_android.h>
 #include <ufo/gralloc.h>
 #include <ufo/graphics.h>
-#elif __ENABLE_WAYLAND__
+#elif defined(__ENABLE_WAYLAND__)
 #include <va/va_wayland.h>
 #endif
 
-#if ANDROID
+#ifdef ANDROID
 #if !defined(EFD_SEMAPHORE)
 #define EFD_SEMAPHORE (1 << 0)
 #endif
@@ -83,13 +83,13 @@ typedef SharedPtr < V4l2CodecBase > V4l2CodecPtr;
 V4l2CodecBase::V4l2CodecBase()
     : m_memoryType(VIDEO_DATA_MEMORY_TYPE_RAW_COPY)
     , m_started(false)
-#if ANDROID
+#ifdef ANDROID
     , m_reqBuffCnt(0)
-#elif(__ENABLE_WAYLAND__ || __ENABLE_X11__)
+#elif defined(__ENABLE_WAYLAND__) || defined(__ENABLE_X11__)
     , m_Display(NULL)
 #else
-    , m_drmfd(0)
 #endif
+    , m_drmfd(0)
     , m_hasEvent(false)
     , m_inputThreadCond(m_frameLock[INPUT])
     , m_outputThreadCond(m_frameLock[OUTPUT])
@@ -112,7 +112,7 @@ V4l2CodecBase::V4l2CodecBase()
     m_frameCount[OUTPUT] = 0;
 #endif
 
-#if ANDROID
+#ifdef ANDROID
     // it's better done inside vaapi driver (retrieve buffer picth etc)
     hw_get_module(GRALLOC_HARDWARE_MODULE_ID, (hw_module_t const**)&m_pGralloc);
     ASSERT(m_pGralloc);
@@ -190,6 +190,18 @@ void V4l2CodecBase::workerThread()
     }
 
     while (m_streamOn[thread]) {
+        {
+            AutoLock locker(m_frameLock[thread]);
+            //ack i/o thread that it is safe to release the buffer queue
+            if (m_reqBufState[thread] == RBS_Request) {
+                m_reqBufState[thread] = RBS_Acknowledge;
+                m_threadCond[thread]->signal();
+            }
+            // wait until i/o thead has released the buffer queue
+            while (m_reqBufState[thread] == RBS_Acknowledge) {
+                m_threadCond[thread]->wait();
+            }
+        }
         {
             AutoLock locker(m_frameLock[thread]);
             if (m_framesTodo[thread].empty()) {
@@ -242,17 +254,6 @@ void V4l2CodecBase::workerThread()
                 DEBUG("%s thread wait because operation on yami fails", THREAD_NAME(thread));
                 m_threadCond[thread]->wait(); // wait if encode/getOutput fail (encode hw is busy or no available output)
             }
-
-            //ack i/o thread that it is safe to release the buffer queue
-            if (m_reqBufState[thread] == RBS_Request) {
-                m_reqBufState[thread] = RBS_Acknowledge;
-                m_threadCond[thread]->signal();
-            }
-            // wait until i/o thead has released the buffer queue
-            while (m_reqBufState[thread] == RBS_Acknowledge) {
-                m_threadCond[thread]->wait();
-            }
-
         }//protected by mLock
         DEBUG("fd: %d", m_fd[0]);
     }
@@ -279,7 +280,7 @@ static void* _workerThread(void *arg)
     return NULL;
 }
 
-#if __ENABLE_DEBUG__
+#if defined(__ENABLE_DEBUG__)
 const char* V4l2CodecBase::IoctlCommandString(int command)
 {
     static const char* unknown = "Unkonwn command";
@@ -362,6 +363,7 @@ int32_t V4l2CodecBase::ioctl(int command, void* arg)
             while (m_threadOn[port]) {
                 if (port == INPUT) {
                     DEBUG("INPUT port got STREAMOFF, release internal lock");
+                    setEosState(EosStateNormal);
                     releaseCodecLock(false);
                 }
                 DEBUG("%s port got STREAMOFF, wait until the worker thread exit/cleanup", THREAD_NAME(port));
@@ -402,6 +404,8 @@ int32_t V4l2CodecBase::ioctl(int command, void* arg)
 
                 // info work thread that we want to release the buffer queue
                 m_reqBufState[port] = RBS_Request;
+                // in case work thread is locked by yami codec
+                releaseCodecLock(false);
 
                 //try to wakeup workthread (workthread may not be active, after EOS for example)
                 m_threadCond[port]->signal();
@@ -410,6 +414,9 @@ int32_t V4l2CodecBase::ioctl(int command, void* arg)
                 while (m_reqBufState[port] != RBS_Acknowledge) {
                     m_threadCond[port]->wait();
                 }
+                // bring yami codec back to normal
+                releaseCodecLock(true);
+
                 DEBUG("m_framesTodo[%d].size %zu, m_framesDone[%d].size %zu\n",
                     port, m_framesTodo[port].size(), port, m_framesDone[port].size());
                 m_framesTodo[port].clear();
@@ -465,7 +472,7 @@ int32_t V4l2CodecBase::ioctl(int command, void* arg)
             #endif
             }
 
-            DEBUG("port: %d, m_framesDone[port].size(): %zu, m_reqBufState[port]: %d",
+            DEBUG("m_framesDone[%d].size(): %zu, m_reqBufState[port]: %d",
                 port, m_framesDone[port].size(), m_reqBufState[port]);
             {
                 AutoLock locker (m_frameLock[port]);
@@ -637,7 +644,25 @@ const char* mimeFromV4l2PixelFormat(uint32_t pixelFormat)
     return mime;
 }
 
-#if ANDROID
+class VideoFrameDeleter {
+public:
+    VideoFrameDeleter(VADisplay display)
+        : m_display(display)
+    {
+    }
+    void operator()(VideoFrame* frame)
+    {
+        VASurfaceID id = (VASurfaceID)frame->surface;
+        DEBUG("destroy VASurface 0x%x", id);
+        vaDestroySurfaces(m_display, &id, 1);
+        delete frame;
+    }
+
+private:
+    VADisplay m_display;
+};
+
+#ifdef ANDROID
 bool V4l2CodecBase::setVaDisplay()
 {
     unsigned int display = ANDROID_DISPLAY;
@@ -707,7 +732,7 @@ SharedPtr<VideoFrame> V4l2CodecBase::createVaSurface(const buffer_handle_t buf_h
     if (vaStatus != VA_STATUS_SUCCESS)
         return frame;
 
-    frame.reset(new VideoFrame);
+    frame.reset(new VideoFrame, VideoFrameDeleter(m_vaDisplay));
     memset(frame.get(), 0, sizeof(VideoFrame));
 
     frame->surface = static_cast<intptr_t>(id);
@@ -730,7 +755,7 @@ bool V4l2CodecBase::mapVideoFrames(int32_t width, int32_t height)
     }
     return true;
 }
-#elif __ENABLE_WAYLAND__
+#elif defined(__ENABLE_WAYLAND__)
 bool V4l2CodecBase::setVaDisplay()
 {
     VAStatus status;
@@ -767,7 +792,7 @@ SharedPtr<VideoFrame> V4l2CodecBase::createVaSurface(uint32_t width, uint32_t he
     if (vaStatus != VA_STATUS_SUCCESS)
         return frame;
 
-    frame.reset(new VideoFrame);
+    frame.reset(new VideoFrame, VideoFrameDeleter(m_vaDisplay));
     memset(frame.get(), 0, sizeof(VideoFrame));
     frame->surface = static_cast<intptr_t>(id);
     frame->crop.width = width;
