@@ -30,27 +30,35 @@
 #ifdef ANDROID
 #include <ufo/gralloc.h>
 #include <ufo/graphics.h>
-#elif __ENABLE_WAYLAND__
+#elif defined(__ENABLE_WAYLAND__)
 #include <va/va_wayland.h>
 #endif
 #include "v4l2_decode.h"
 #include "VideoDecoderHost.h"
 #include "common/log.h"
+#if defined(__ENABLE_EGL__)
 #include "egl/egl_vaapi_image.h"
+#endif
 
-#define INT64_TO_TIMEVAL(i64, time_val) do {            \
-        time_val.tv_sec = (int32_t)(i64 >> 32);         \
-        time_val.tv_usec = (int32_t)(i64 & 0xffffffff); \
+#define INT64_TO_TIMEVAL(i64, time_val)                 \
+    do {                                                \
+        time_val.tv_sec = (int32_t)(i64 >> 31);         \
+        time_val.tv_usec = (int32_t)(i64 & 0x7fffffff); \
+    } while (0)
+#define TIMEVAL_TO_INT64(i64, time_val)       \
+    do {                                      \
+        i64 = time_val.tv_sec;                \
+        i64 = (i64 << 31) + time_val.tv_usec; \
     } while (0)
-#define TIMEVAL_TO_INT64(i64, time_val) do {            \
-        i64 = time_val.tv_sec;                          \
-        i64 = (i64 << 32) + time_val.tv_usec;           \
-    } while(0)
 
 V4l2Decoder::V4l2Decoder()
     : m_bindEglImage(false)
     , m_videoWidth(0)
     , m_videoHeight(0)
+    , m_outputBufferCountOnInit(0)
+    , m_outputBufferCountQBuf(0)
+    , m_outputBufferCountPulse(0)
+    , m_outputBufferCountGive(0)
 {
     uint32_t i;
     m_memoryMode[INPUT] = V4L2_MEMORY_MMAP; // dma_buf hasn't been supported yet
@@ -120,26 +128,26 @@ bool V4l2Decoder::start()
 #endif
 
     NativeDisplay nativeDisplay;
+    nativeDisplay.type = NATIVE_DISPLAY_AUTO;
 
-#if ANDROID
-    nativeDisplay.type = NATIVE_DISPLAY_VA;
-    nativeDisplay.handle = (intptr_t)m_vaDisplay;
-#elif __ENABLE_WAYLAND__
-    nativeDisplay.type = NATIVE_DISPLAY_VA;
-    nativeDisplay.handle = (intptr_t)m_vaDisplay;
-#elif __ENABLE_X11__
+#if defined(ANDROID) || defined(__ENABLE_WAYLAND__)
+    if (m_vaDisplay) {
+        nativeDisplay.type = NATIVE_DISPLAY_VA;
+        nativeDisplay.handle = (intptr_t)m_vaDisplay;
+    }
+#elif defined(__ENABLE_X11__)
     DEBUG("m_Display: %p", m_Display);
     if (m_Display) {
         nativeDisplay.type = NATIVE_DISPLAY_X11;
         nativeDisplay.handle = (intptr_t)m_Display;
-    } else {
+    }
+#endif
+
+    if (m_drmfd) {
         nativeDisplay.type = NATIVE_DISPLAY_DRM;
         nativeDisplay.handle = m_drmfd;
     }
-#else
-    nativeDisplay.type = NATIVE_DISPLAY_DRM;
-    nativeDisplay.handle = m_drmfd;
-#endif
+
     m_decoder->setNativeDisplay(&nativeDisplay);
 
     // send codec_data if there is
@@ -211,10 +219,12 @@ bool V4l2Decoder::outputPulse(uint32_t &index)
     m_vpp->process(output, m_videoFrames[index]);
     m_videoFrames[index]->timeStamp = output->timeStamp;
     m_videoFrames[index]->flags = output->flags;
-    DEBUG("buffer index: %d, timeStamp: %" PRId64 "\n", index, output->timeStamp);
+    DEBUG("m_outputBufferCountPulse: %d, buffer index: %d, surface: %p, timeStamp: %" PRId64 "\n",
+        m_outputBufferCountPulse, index, (void*)m_videoFrames[index]->surface, output->timeStamp);
+    m_outputBufferCountPulse++;
     return true;
 }
-#else
+#elif __ENABLE_EGL__
 bool V4l2Decoder::outputPulse(uint32_t &index)
 {
     SharedPtr<VideoFrame> frame;
@@ -311,16 +321,32 @@ bool V4l2Decoder::acceptInputBuffer(struct v4l2_buffer *qbuf)
 
 bool V4l2Decoder::giveOutputBuffer(struct v4l2_buffer *dqbuf)
 {
-    int i;
     ASSERT(dqbuf);
     // for the buffers within range of [m_actualOutBufferCount, m_maxBufferCount[OUTPUT]]
     // there are not used in reality, but still be returned back to client during flush (seek/eos)
     ASSERT(dqbuf->index < m_maxBufferCount[OUTPUT]);
 
     dqbuf->flags = m_outputRawFrames[dqbuf->index].flags;
+
+#if defined(ANDROID)
+    INT64_TO_TIMEVAL(m_videoFrames[dqbuf->index]->timeStamp, dqbuf->timestamp);
+    dqbuf->flags = m_videoFrames[dqbuf->index]->flags;
+#elif defined(__ENABLE_WAYLAND__)
+    VAStatus vaStatus;
+    struct wl_buffer* buffer;
+    INT64_TO_TIMEVAL(m_videoFrames[dqbuf->index]->timeStamp, dqbuf->timestamp);
+    dqbuf->flags = m_videoFrames[dqbuf->index]->flags;
+    vaStatus = vaGetSurfaceBufferWl(m_vaDisplay, m_videoFrames[dqbuf->index]->surface, VA_FRAME_PICTURE, &buffer);
+    DEBUG("m_outputBufferCountGive: %d, index: %d, surface: %p, wl_buffer: %p", m_outputBufferCountGive, dqbuf->index, (void*)m_videoFrames[dqbuf->index]->surface, buffer);
+    m_outputBufferCountGive++;
+    if (vaStatus != VA_STATUS_SUCCESS)
+        return false;
+    dqbuf->m.userptr = (unsigned long)buffer;
+#else
+    int i;
     if (!m_bindEglImage) {
         // FIXME: m_bufferPlaneCount[OUTPUT] doesn't match current RGBX output, it is a bug
-        for (i=0; i< 1; i++) {
+        for (i = 0; i < 1; i++) {
             // FIXME, a better field to hold drm/dma handle
             dqbuf->m.planes[i].reserved[0] = m_outputRawFrames[dqbuf->index].handle;
             dqbuf->m.planes[i].reserved[1] = m_outputRawFrames[dqbuf->index].pitch[i];
@@ -330,20 +356,7 @@ bool V4l2Decoder::giveOutputBuffer(struct v4l2_buffer *dqbuf)
 
     // simple set size data to satify chrome even in texture mode
     dqbuf->m.planes[0].bytesused = m_videoWidth * m_videoHeight;
-    dqbuf->m.planes[1].bytesused = m_videoWidth * m_videoHeight/2;
-#if ANDROID
-    INT64_TO_TIMEVAL(m_videoFrames[dqbuf->index]->timeStamp, dqbuf->timestamp);
-    dqbuf->flags = m_videoFrames[dqbuf->index]->flags;
-#elif __ENABLE_WAYLAND__
-    VAStatus vaStatus;
-    struct wl_buffer* buffer;
-    INT64_TO_TIMEVAL(m_videoFrames[dqbuf->index]->timeStamp, dqbuf->timestamp);
-    dqbuf->flags = m_videoFrames[dqbuf->index]->flags;
-    vaStatus = vaGetSurfaceBufferWl(m_vaDisplay, m_videoFrames[dqbuf->index]->surface, VA_FRAME_PICTURE, &buffer);
-    if (vaStatus != VA_STATUS_SUCCESS)
-        return false;
-    dqbuf->m.userptr = (unsigned long)buffer;
-#else
+    dqbuf->m.planes[1].bytesused = m_videoWidth * m_videoHeight / 2;
     INT64_TO_TIMEVAL(m_outputRawFrames[dqbuf->index].timeStamp, dqbuf->timestamp);
     dqbuf->flags = m_outputRawFrames[dqbuf->index].flags;
 #endif
@@ -365,33 +378,43 @@ int32_t V4l2Decoder::ioctl(int command, void* arg)
     DEBUG("fd: %d, ioctl command: %s", m_fd[0], IoctlCommandString(command));
     switch (command) {
     case VIDIOC_QBUF: {
-#ifdef ANDROID
         struct v4l2_buffer *qbuf = static_cast<struct v4l2_buffer*>(arg);
-        static uint32_t bufferCount = 0;
+#ifdef ANDROID
         if(qbuf->type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE &&
            m_streamOn[OUTPUT] == false) {
             ASSERT(qbuf->memory == V4L2_MEMORY_ANDROID_BUFFER_HANDLE);
             m_bufferHandle.push_back((buffer_handle_t)(qbuf->m.userptr));
-            bufferCount++;
-            if (bufferCount == m_reqBuffCnt)
+            m_outputBufferCountOnInit++;
+            if (m_outputBufferCountOnInit == m_reqBuffCnt)
                 mapVideoFrames(m_videoWidth, m_videoHeight);
         }
-#elif __ENABLE_WAYLAND__
-        struct v4l2_buffer* qbuf = static_cast<struct v4l2_buffer*>(arg);
-        static uint32_t bufferCount = 0;
+#elif defined(__ENABLE_WAYLAND__)
+        // FIXME, m_outputBufferCountOnInit should be reset on output buffer change (for example: resolution change)
+        // it is not must to init video frame here since we don't accepted external buffer for wayland yet.
+        // however, external buffer may be used in the future
         if (qbuf->type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE && m_streamOn[OUTPUT] == false) {
-            bufferCount++;
-            if (bufferCount == m_reqBuffCnt) {
+            m_outputBufferCountOnInit++;
+            DEBUG("m_outputBufferCountOnInit: %d", m_outputBufferCountOnInit);
+            if (m_outputBufferCountOnInit == m_reqBuffCnt) {
                 mapVideoFrames(m_videoWidth, m_videoHeight);
             }
         }
 #endif
+        if (qbuf->type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE) {
+            DEBUG("m_outputBufferCountQBuf: %d, buffer index: %d", m_outputBufferCountQBuf, qbuf->index);
+            m_outputBufferCountQBuf++;
+        }
     } // no break;
     case VIDIOC_STREAMON:
     case VIDIOC_STREAMOFF:
     case VIDIOC_DQBUF:
     case VIDIOC_QUERYCAP:
         ret = V4l2CodecBase::ioctl(command, arg);
+        if (command == VIDIOC_STREAMON) {
+            m_outputBufferCountQBuf = 0;
+            m_outputBufferCountPulse = 0;
+            m_outputBufferCountGive = 0;
+        }
         break;
     case VIDIOC_REQBUFS: {
         ret = V4l2CodecBase::ioctl(command, arg);
@@ -405,7 +428,7 @@ int32_t V4l2Decoder::ioctl(int command, void* arg)
                 m_reqBuffCnt = reqbufs->count;
             else
                 m_videoFrames.clear();
-        #else
+#elif defined(__ENABLE_EGL__)
             if (!reqbufs->count) {
                 m_eglVaapiImages.clear();
             } else {
@@ -423,7 +446,7 @@ int32_t V4l2Decoder::ioctl(int command, void* arg)
                     m_eglVaapiImages.push_back(image);
                 }
             }
-        #endif
+#endif
         }
         break;
     }
@@ -531,9 +554,7 @@ int32_t V4l2Decoder::ioctl(int command, void* arg)
             m_videoHeight = outFormat->height;
             m_maxBufferSize[OUTPUT] = m_videoWidth * m_videoHeight + ((m_videoWidth +1)/2*2) * ((m_videoHeight+1)/2);
         } else {
-            ret = -1;
-            // chromeos accepts EINVAL as not enough input data yet, will try it again.
-            errno = EINVAL;
+            ret = EAGAIN;
         }
       }
     break;
@@ -641,7 +662,7 @@ void V4l2Decoder::flush()
         m_decoder->flush();
 }
 
-#if (!defined(ANDROID) && !defined(__ENABLE_WAYLAND__))
+#ifdef __ENABLE_EGL__
 int32_t V4l2Decoder::useEglImage(EGLDisplay eglDisplay, EGLContext eglContext, uint32_t bufferIndex, void* eglImage)
 {
     m_bindEglImage = true;
